﻿using HarmonyLib;
using System.Linq;
using UnityEngine;

namespace DesalinatorsPlus
{
	/// <summary>
	/// This would add the Uranium Ore byproduct from Salt Water
	/// filtration and desalination process.
	/// </summary>
	[HarmonyPatch(typeof(DesalinatorConfig), "ConfigureBuildingTemplate")]
	public class DesalinatorConfig_ConfigureBuildingTemplate
	{
		/// <summary>
		/// The Harmony patch of the 'DesalinatorConfig::ConfigureBuildingTemplate' method.
		/// </summary>
		/// <see cref="DesalinatorConfig.ConfigureBuildingTemplate(GameObject, Tag)"/>
		/// <param name="go">The 'DesalinatorConfig' itself.</param>
		public static void Postfix(GameObject go)
		{
			// Gimmet dat 'ElementConverter' for dis Desalinator, boi!
			var elemConverters = go.GetComponents<ElementConverter>();
			// Got some Converters, mate?
			if (elemConverters != null && elemConverters.Any())
			{
				foreach (var elemConv in elemConverters)
				{
					// No OutputElements == !!error!!
					// So we're not moving forth.
					if (elemConv.outputElements != null && elemConv.outputElements.Any())
					{
						//TODO: instead of allocating a new List<> it would be better, resources-wise (but a little less KISSey), to instantiate an Array that shall be assigned without further gimmicks or conversions.
						var outElems = elemConv.outputElements
											.AsParallel() // <- are you sure is it really needed, Mrs. Blinkermeier?
											.ToList();
						// Eat some more memory, kid - ye look so hungry, my poor soul!
						var uraniumOre = new ElementConverter.OutputElement(0.01f
								, SimHashes.UraniumOre
								, -30.0f
								, storeOutput: true);
						// Stuff that Uranium Ore Element into Desalinator's
						// output products' list(*).
						outElems.Add(uraniumOre);
						// (*) that list is going to become a shiny Array just... NOW!
						elemConv.outputElements = outElems.ToArray();
						// Voles are red
						// Hatches are blue
						// so eat all of them
						// before they eat you.
					}
					else
					{
						//TODO: maybe log something, aye?
					}
				}
			}
			else
			{
				// TODO: add some diagnostic informations blahblah, maybe.
			}
		}
	}
	/// <summary>
	/// When the Desalinator is going to be emptied, drop the Uranium Ore
	/// first, in order to leave almost untouched
	/// the original method... and the original machine's Storage, of course.
	/// </summary>
	[HarmonyPatch(typeof(Desalinator.StatesInstance), "OnEmptyComplete")]
	public class Desalinator_StatesInstance_OnEmptyComplete
	{
		/// <summary>
		/// The patch to eject Uranium Ore together with Salt!
		/// </summary>
		/// <see cref="Desalinator.StatesInstance"/>
		/// <param name="__instance">Equivalent to 'this' from the gut of the 'Desalinator.StatesInstance' object.</param>
		public static void Prefix(Desalinator.StatesInstance __instance)
		{
			// Reality checks - MIMIMIMIMI...
			if (__instance == null || __instance.master == null)
				return;
			// I don't know why I'm wrapping the following mess
			// inside a shameful try-catch block.
			// Maybe I've grown too old and forethoughtful, that's it.
			try
			{
				// Oh mighty Klei's God, have mercy of this stinky dog, plz!
				// Starts juggling with Reflection in order to theft
				// the Desalinator's storage data.
				var storageField = typeof(Desalinator)
						.GetField("storage"
								, System.Reflection.BindingFlags.NonPublic
										| System.Reflection.BindingFlags.Instance);
				// Well...
				if (storageField == null)
					return;

				var storage = storageField.GetValue(__instance.master) as Storage;
				// ...let's remember that Blind Faith Programming 
				// is an anti-pattern, after all.
				// Oof.
				// And there it comes again.
				if (storage == null)
					return;
				// Gib Uranium Ore ID!
				var tag = GameTagExtensions.Create(SimHashes.UraniumOre);
				// And do some trickery black magic under the hood, ofc.
				var result = ListPool<GameObject, Desalinator>.Allocate();
				// Hey mister, got some Uranium to be sold in your grocery?
				storage.Find(tag, result);
				// Shhh, just some un-useful Parallelism finesse.
				foreach (GameObject go in result.AsParallel())
					storage.Drop(go);
				// Greenwashing anyone?
				result.Recycle();
				// Oh well, it seems that we're still alive, after a-
			}
			catch
			{
				// TODO: as in the 'DesalinatorConfig_ConfigureBuildingTemplate::Postfix', perhaps write some logs here.
			}
		}
	}
}
